/* xrw2pdf.c
 * 
 * David Barnes, September 2011
 *
 * Distributed as part of the S2PLOT library.
 *
 * Requires the S2PLOT PRC export module.
 *
 * ==================================================================
 * S2PLOT is freely available for non-commercial and educational use,
 * however, it is not public-domain software. The programming library
 * and documentation may not be redistributed or sub-licensed in any
 * form without permission from Swinburne University of
 * Technology. The programming library and sample codes are provided
 * without warranty.
 *
 * We would appreciate it if research outcomes using S2PLOT would
 * provide the following acknowledgement:
 *
 * "Three-dimensional visualisation was conducted with the S2PLOT
 * progamming library"
 *
 * and a reference to
 *
 * D.G.Barnes, C.J.Fluke, P.D.Bourke & O.T.Parry, 2006, Publications
 * of the Astronomical Society of Australia, 23(2), 82-93.
 *
 * S2PLOT development was funded through the Researcher Developer
 * Grant Scheme at Swinburne University of Technology.
 * ==================================================================
 *
 * $Id:$
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include "s2plot.h"
#include "libxrw.c"

void ds2dvrXXX(int, int, int);
int vidx, vidy, vidz;
void cb(double *t, int *kc) {
  pushVRMLname("VOLRENDER");
  pushVRMLname("VRSET1");
  ds2dvrXXX(vidx, 0, 1);
  pushVRMLname("VRSET2");
  ds2dvrXXX(vidy, 0, 2);
  pushVRMLname("VRSET3");
  ds2dvrXXX(vidz, 0, 3);
  pushVRMLname("ANON");
  pushVRMLname("ANON");
}

float dmin, dmax, amin, amax;		/* Min/max values */
float alphafn(float *dval) {
  float valsca = (*dval - dmin)  / (dmax - dmin);
  valsca = valsca < 0.0 ? 0.0 : valsca > 1.0 ? 1.0 : valsca;
  return amin + (amax - amin) * powf(valsca, 0.33);
}

int main(int argc, char *argv[]) {

  if (argc != 2) {
    fprintf(stderr, "usage: %s xrwfilename\n", argv[0]);
    return -1;
  }

  XRAW_STRUCT *xr = loadXraw(argv[1]);
  if (!xr) {
    fprintf(stderr, "Failed to open or read '%s'.\n", argv[1]);
    return -1;
  }

  showXraw(xr);
  fprintf(stdout, "- - - - - - - - - - - - - - - - - - - - - - - - -\n");

  int stride[3] = {4,4,2};
  VOL_STRUCT *vol = parseXraw(xr, stride);
  if (!vol) {
    fprintf(stderr, "Failed to parse data volume.\n");
    return -1;
  }

  showXvol(vol);

  int nx, ny, nz;
  nx = vol->nx;
  ny = vol->ny;
  nz = vol->nz;
  
  s2opend("/S2MONO",argc,argv);			/* Open the display */
  
  // viewport: simply generate appropriate aspect ratio
  float sx = 1.0;
  float sy = vol->wdy*(float)ny / (vol->wdx*(float)nx);
  float sz = vol->wdz*(float)nz / (vol->wdx*(float)nx);
  s2svp(-sx,sx, -sy,sy, -sz,sz);

  // world coordinate system: put centre of volume at (0,0,0)
  // world coordinates are physical units
  float ddx = vol->wdx*(float)nx * 0.5;
  float ddy = vol->wdy*(float)ny * 0.5;
  float ddz = vol->wdz*(float)nz * 0.5;
  s2swin(-ddx,ddx, -ddy,ddy, -ddz,ddz);
  //s2swin(-vol->wdx*(float)nx*0.5, vol->wdx*(float)nx*0.5,
  //	 -vol->wdy*(float)ny*0.5, vol->wdy*(float)ny*0.5,
  //	 -vol->wdz*(float)nz*0.5, vol->wdz*(float)nz*0.5);

  // set up lights
  COLOUR ambient = {0.7,0.7,0.7};
  XYZ lightpos[2];
  COLOUR lightcol[2];
  lightpos[0].x = -ddx*2;
  lightpos[0].y = -ddy*2;
  lightpos[0].z = -ddz*2;
  lightpos[1].x = ddx*2;
  lightpos[1].y = ddy*2;
  lightpos[1].z = ddy*2;
  lightcol[0].r = lightcol[0].g = lightcol[0].b = 1.0;
  lightcol[1].r = lightcol[1].g = lightcol[1].b = 1.0;
  ss2sl(ambient, 2, lightpos, lightcol, 1);

  s2box("BCDE",0,0,"BCDE",0,0,"BCDE",0,0);		/* Draw coord box */  

  int i; 					/* Loop variable */
  
  float tr[12];
  for (i=0;i<12;i++) {				/* Set-up transfrom matrix */
    tr[i] = 0.0;
  }
  tr[0] = -vol->wdx*(float)nx*0.5;
  tr[1] = vol->wdx;
  tr[4] = -vol->wdy*(float)ny*0.5;
  tr[6] = vol->wdy;
  tr[8] = -vol->wdz*(float)nz*0.5;
  tr[11]= vol->wdz;

  dmin = 0.0;
  dmax = 1.0;			/* Maximum data value in volume to plot */
  amin = 0.0;			/* Minimum alpha channel value */
  amax = 0.1;			/* Maximum alpha channel value */
  char trans = 't';			/* Transparency type */
 

  // install colour LUT
  s2scir(1000, 1255);
  for (i = 0; i < 256; i++) {
    s2scr(1000+i, vol->red[i], vol->green[i], vol->blue[i]);
  }
 
  if (1) {
  vidx = ns2cvra(vol->data, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1, 
		 tr, trans, dmin, dmax, alphafn);
  vidy = ns2cvra(vol->data, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1, 
		 tr, trans, dmin, dmax, alphafn);
  vidz = ns2cvra(vol->data, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1, 
		 tr, trans, dmin, dmax, alphafn);
  cs2scb(cb);				/* Install a dynamic callback */
  }
  
  // high-quality surface = smooth normals
  ns2ssq(1);
  int isoid = ns2cis(vol->data, nx, ny, nz, 0, nx-1, 0, ny-1, 0, nz-1, 
		     tr, 0.3, 2, 'o', 1.0, 0.3, 0.7, 1.0);
  pushVRMLname("ISOSURFACE");
  ns2dis(isoid, 0);
  pushVRMLname("ANON");
 
  //ss2srm(SHADE_FLAT);			/* Set shading type to FLAT */
  
  s2show(1);				/* Open the s2plot window */
  
  return 1;
}


